@@ -265,28 +265,28 @@ static int snd_ctl_hw_read(snd_ctl_t *handle, snd_ctl_event_t *event)
 }
 
 snd_ctl_ops_t snd_ctl_hw_ops = {
-	close: snd_ctl_hw_close,
-	nonblock: snd_ctl_hw_nonblock,
-	async: snd_ctl_hw_async,
-	subscribe_events: snd_ctl_hw_subscribe_events,
-	card_info: snd_ctl_hw_card_info,
-	element_list: snd_ctl_hw_elem_list,
-	element_info: snd_ctl_hw_elem_info,
-	element_read: snd_ctl_hw_elem_read,
-	element_write: snd_ctl_hw_elem_write,
-	element_lock: snd_ctl_hw_elem_lock,
-	element_unlock: snd_ctl_hw_elem_unlock,
-	hwdep_next_device: snd_ctl_hw_hwdep_next_device,
-	hwdep_info: snd_ctl_hw_hwdep_info,
-	pcm_next_device: snd_ctl_hw_pcm_next_device,
-	pcm_info: snd_ctl_hw_pcm_info,
-	pcm_prefer_subdevice: snd_ctl_hw_pcm_prefer_subdevice,
-	rawmidi_next_device: snd_ctl_hw_rawmidi_next_device,
-	rawmidi_info: snd_ctl_hw_rawmidi_info,
-	rawmidi_prefer_subdevice: snd_ctl_hw_rawmidi_prefer_subdevice,
-	set_power_state: snd_ctl_hw_set_power_state,
-	get_power_state: snd_ctl_hw_get_power_state,
-	read: snd_ctl_hw_read,
+	.close = snd_ctl_hw_close,
+	.nonblock = snd_ctl_hw_nonblock,
+	.async = snd_ctl_hw_async,
+	.subscribe_events = snd_ctl_hw_subscribe_events,
+	.card_info = snd_ctl_hw_card_info,
+	.element_list = snd_ctl_hw_elem_list,
+	.element_info = snd_ctl_hw_elem_info,
+	.element_read = snd_ctl_hw_elem_read,
+	.element_write = snd_ctl_hw_elem_write,
+	.element_lock = snd_ctl_hw_elem_lock,
+	.element_unlock = snd_ctl_hw_elem_unlock,
+	.hwdep_next_device = snd_ctl_hw_hwdep_next_device,
+	.hwdep_info = snd_ctl_hw_hwdep_info,
+	.pcm_next_device = snd_ctl_hw_pcm_next_device,
+	.pcm_info = snd_ctl_hw_pcm_info,
+	.pcm_prefer_subdevice = snd_ctl_hw_pcm_prefer_subdevice,
+	.rawmidi_next_device = snd_ctl_hw_rawmidi_next_device,
+	.rawmidi_info = snd_ctl_hw_rawmidi_info,
+	.rawmidi_prefer_subdevice = snd_ctl_hw_rawmidi_prefer_subdevice,
+	.set_power_state = snd_ctl_hw_set_power_state,
+	.get_power_state = snd_ctl_hw_get_power_state,
+	.read = snd_ctl_hw_read,
 };
 
 int snd_ctl_hw_open(snd_ctl_t **handle, const char *name, int card, int mode)
