@@ -600,41 +600,41 @@ static void snd_pcm_jack_dump(snd_pcm_t *pcm, snd_output_t *out)
 }
 
 static snd_pcm_ops_t snd_pcm_jack_ops = {
-	close: snd_pcm_jack_close,
-	info: snd_pcm_jack_info,
-	hw_refine: snd_pcm_jack_hw_refine,
-	hw_params: snd_pcm_jack_hw_params,
-	hw_free: snd_pcm_jack_hw_free,
-	sw_params: snd_pcm_jack_sw_params,
-	channel_info: snd_pcm_jack_channel_info,
-	dump: snd_pcm_jack_dump,
-	nonblock: snd_pcm_jack_nonblock,
-	async: snd_pcm_jack_async,
-	poll_revents: snd_pcm_jack_poll_revents,
-	mmap: snd_pcm_jack_mmap,
-	munmap: snd_pcm_jack_munmap,
+	.close = snd_pcm_jack_close,
+	.info = snd_pcm_jack_info,
+	.hw_refine = snd_pcm_jack_hw_refine,
+	.hw_params = snd_pcm_jack_hw_params,
+	.hw_free = snd_pcm_jack_hw_free,
+	.sw_params = snd_pcm_jack_sw_params,
+	.channel_info = snd_pcm_jack_channel_info,
+	.dump = snd_pcm_jack_dump,
+	.nonblock = snd_pcm_jack_nonblock,
+	.async = snd_pcm_jack_async,
+	.poll_revents = snd_pcm_jack_poll_revents,
+	.mmap = snd_pcm_jack_mmap,
+	.munmap = snd_pcm_jack_munmap,
 };
 
 static snd_pcm_fast_ops_t snd_pcm_jack_fast_ops = {
-	status: snd_pcm_jack_status,
-	state: snd_pcm_jack_state,
-	hwsync: snd_pcm_jack_hwsync,
-	delay: snd_pcm_jack_delay,
-	prepare: snd_pcm_jack_prepare,
-	reset: snd_pcm_jack_reset,
-	start: snd_pcm_jack_start,
-	drop: snd_pcm_jack_drop,
-	drain: snd_pcm_jack_drain,
-	pause: snd_pcm_jack_pause,
-	rewind: snd_pcm_jack_rewind,
-	forward: snd_pcm_jack_forward,
-	resume: snd_pcm_jack_resume,
-	writei: snd_pcm_mmap_writei,
-	writen: snd_pcm_mmap_writen,
-	readi: snd_pcm_mmap_readi,
-	readn: snd_pcm_mmap_readn,
-	avail_update: snd_pcm_jack_avail_update,
-	mmap_commit: snd_pcm_jack_mmap_commit,
+	.status = snd_pcm_jack_status,
+	.state = snd_pcm_jack_state,
+	.hwsync = snd_pcm_jack_hwsync,
+	.delay = snd_pcm_jack_delay,
+	.prepare = snd_pcm_jack_prepare,
+	.reset = snd_pcm_jack_reset,
+	.start = snd_pcm_jack_start,
+	.drop = snd_pcm_jack_drop,
+	.drain = snd_pcm_jack_drain,
+	.pause = snd_pcm_jack_pause,
+	.rewind = snd_pcm_jack_rewind,
+	.forward = snd_pcm_jack_forward,
+	.resume = snd_pcm_jack_resume,
+	.writei = snd_pcm_mmap_writei,
+	.writen = snd_pcm_mmap_writen,
+	.readi = snd_pcm_mmap_readi,
+	.readn = snd_pcm_mmap_readn,
+	.avail_update = snd_pcm_jack_avail_update,
+	.mmap_commit = snd_pcm_jack_mmap_commit,
 };
 
 static int parse_ports(snd_config_t *conf,char*** ret_ports,int *ret_n)
