@@ -280,6 +280,7 @@ static void server_job(snd_pcm_direct_t *dmix)
 	close(dmix->hw_fd);
 	if (dmix->server_free)
 		dmix->server_free(dmix);
+	unlink(dmix->shmptr->socket_name);
 	snd_pcm_direct_shm_discard(dmix);
 	snd_pcm_direct_semaphore_discard(dmix);
 	server_printf("DIRECT SERVER EXIT\n");
@@ -312,7 +313,12 @@ int snd_pcm_direct_server_create(snd_pcm_direct_t *dmix)
 		close(dmix->server_fd);
 		return ret;
 	} else if (ret == 0) {
-		server_job(dmix);
+		ret = fork();
+		if (ret == 0)
+			server_job(dmix);
+		exit(EXIT_SUCCESS);
+	} else {
+		waitpid(ret, NULL, 0);
 	}
 	dmix->server_pid = ret;
 	dmix->server = 1;
