@@ -832,41 +832,41 @@ static void snd_pcm_dmix_dump(snd_pcm_t *pcm, snd_output_t *out)
 }
 
 static snd_pcm_ops_t snd_pcm_dmix_ops = {
-	close: snd_pcm_dmix_close,
-	info: snd_pcm_dmix_info,
-	hw_refine: snd_pcm_dmix_hw_refine,
-	hw_params: snd_pcm_dmix_hw_params,
-	hw_free: snd_pcm_dmix_hw_free,
-	sw_params: snd_pcm_dmix_sw_params,
-	channel_info: snd_pcm_dmix_channel_info,
-	dump: snd_pcm_dmix_dump,
-	nonblock: snd_pcm_dmix_nonblock,
-	async: snd_pcm_dmix_async,
-	poll_revents: snd_pcm_dmix_poll_revents,
-	mmap: snd_pcm_dmix_mmap,
-	munmap: snd_pcm_dmix_munmap,
+	.close = snd_pcm_dmix_close,
+	.info = snd_pcm_dmix_info,
+	.hw_refine = snd_pcm_dmix_hw_refine,
+	.hw_params = snd_pcm_dmix_hw_params,
+	.hw_free = snd_pcm_dmix_hw_free,
+	.sw_params = snd_pcm_dmix_sw_params,
+	.channel_info = snd_pcm_dmix_channel_info,
+	.dump = snd_pcm_dmix_dump,
+	.nonblock = snd_pcm_dmix_nonblock,
+	.async = snd_pcm_dmix_async,
+	.poll_revents = snd_pcm_dmix_poll_revents,
+	.mmap = snd_pcm_dmix_mmap,
+	.munmap = snd_pcm_dmix_munmap,
 };
 
 static snd_pcm_fast_ops_t snd_pcm_dmix_fast_ops = {
-	status: snd_pcm_dmix_status,
-	state: snd_pcm_dmix_state,
-	hwsync: snd_pcm_dmix_hwsync,
-	delay: snd_pcm_dmix_delay,
-	prepare: snd_pcm_dmix_prepare,
-	reset: snd_pcm_dmix_reset,
-	start: snd_pcm_dmix_start,
-	drop: snd_pcm_dmix_drop,
-	drain: snd_pcm_dmix_drain,
-	pause: snd_pcm_dmix_pause,
-	rewind: snd_pcm_dmix_rewind,
-	forward: snd_pcm_dmix_forward,
-	resume: snd_pcm_dmix_resume,
-	writei: snd_pcm_mmap_writei,
-	writen: snd_pcm_mmap_writen,
-	readi: snd_pcm_dmix_readi,
-	readn: snd_pcm_dmix_readn,
-	avail_update: snd_pcm_dmix_avail_update,
-	mmap_commit: snd_pcm_dmix_mmap_commit,
+	.status = snd_pcm_dmix_status,
+	.state = snd_pcm_dmix_state,
+	.hwsync = snd_pcm_dmix_hwsync,
+	.delay = snd_pcm_dmix_delay,
+	.prepare = snd_pcm_dmix_prepare,
+	.reset = snd_pcm_dmix_reset,
+	.start = snd_pcm_dmix_start,
+	.drop = snd_pcm_dmix_drop,
+	.drain = snd_pcm_dmix_drain,
+	.pause = snd_pcm_dmix_pause,
+	.rewind = snd_pcm_dmix_rewind,
+	.forward = snd_pcm_dmix_forward,
+	.resume = snd_pcm_dmix_resume,
+	.writei = snd_pcm_mmap_writei,
+	.writen = snd_pcm_mmap_writen,
+	.readi = snd_pcm_dmix_readi,
+	.readn = snd_pcm_dmix_readn,
+	.avail_update = snd_pcm_dmix_avail_update,
+	.mmap_commit = snd_pcm_dmix_mmap_commit,
 };
 
 /**
@@ -893,6 +893,7 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	snd_pcm_t *pcm = NULL, *spcm = NULL;
 	snd_pcm_direct_t *dmix = NULL;
 	int ret, first_instance;
+	int fail_sem_loop = 10;
 
 	assert(pcmp);
 
@@ -919,16 +920,21 @@ int snd_pcm_dmix_open(snd_pcm_t **pcmp, const char *name,
 	if (ret < 0)
 		goto _err;
 
-	ret = snd_pcm_direct_semaphore_create_or_connect(dmix);
-	if (ret < 0) {
-		SNDERR("unable to create IPC semaphore");
-		goto _err;
-	}
 	
-	ret = snd_pcm_direct_semaphore_down(dmix, DIRECT_IPC_SEM_CLIENT);
-	if (ret < 0) {
-		snd_pcm_direct_semaphore_discard(dmix);
-		goto _err;
+	while (1) {
+		ret = snd_pcm_direct_semaphore_create_or_connect(dmix);
+		if (ret < 0) {
+			SNDERR("unable to create IPC semaphore");
+			goto _err;
+		}
+		ret = snd_pcm_direct_semaphore_down(dmix, DIRECT_IPC_SEM_CLIENT);
+		if (ret < 0) {
+			snd_pcm_direct_semaphore_discard(dmix);
+			if (--fail_sem_loop <= 0)
+				goto _err;
+			continue;
+		}
+		break;
 	}
 		
 	first_instance = ret = snd_pcm_direct_shm_create_or_connect(dmix);
