@@ -622,41 +622,41 @@ static void snd_pcm_dshare_dump(snd_pcm_t *pcm, snd_output_t *out)
 }
 
 static snd_pcm_ops_t snd_pcm_dshare_ops = {
-	close: snd_pcm_dshare_close,
-	info: snd_pcm_dshare_info,
-	hw_refine: snd_pcm_dshare_hw_refine,
-	hw_params: snd_pcm_dshare_hw_params,
-	hw_free: snd_pcm_dshare_hw_free,
-	sw_params: snd_pcm_dshare_sw_params,
-	channel_info: snd_pcm_dshare_channel_info,
-	dump: snd_pcm_dshare_dump,
-	nonblock: snd_pcm_dshare_nonblock,
-	async: snd_pcm_dshare_async,
-	poll_revents: snd_pcm_dshare_poll_revents,
-	mmap: snd_pcm_dshare_mmap,
-	munmap: snd_pcm_dshare_munmap,
+	.close = snd_pcm_dshare_close,
+	.info = snd_pcm_dshare_info,
+	.hw_refine = snd_pcm_dshare_hw_refine,
+	.hw_params = snd_pcm_dshare_hw_params,
+	.hw_free = snd_pcm_dshare_hw_free,
+	.sw_params = snd_pcm_dshare_sw_params,
+	.channel_info = snd_pcm_dshare_channel_info,
+	.dump = snd_pcm_dshare_dump,
+	.nonblock = snd_pcm_dshare_nonblock,
+	.async = snd_pcm_dshare_async,
+	.poll_revents = snd_pcm_dshare_poll_revents,
+	.mmap = snd_pcm_dshare_mmap,
+	.munmap = snd_pcm_dshare_munmap,
 };
 
 static snd_pcm_fast_ops_t snd_pcm_dshare_fast_ops = {
-	status: snd_pcm_dshare_status,
-	state: snd_pcm_dshare_state,
-	hwsync: snd_pcm_dshare_hwsync,
-	delay: snd_pcm_dshare_delay,
-	prepare: snd_pcm_dshare_prepare,
-	reset: snd_pcm_dshare_reset,
-	start: snd_pcm_dshare_start,
-	drop: snd_pcm_dshare_drop,
-	drain: snd_pcm_dshare_drain,
-	pause: snd_pcm_dshare_pause,
-	rewind: snd_pcm_dshare_rewind,
-	forward: snd_pcm_dshare_forward,
-	resume: snd_pcm_dshare_resume,
-	writei: snd_pcm_mmap_writei,
-	writen: snd_pcm_mmap_writen,
-	readi: snd_pcm_dshare_readi,
-	readn: snd_pcm_dshare_readn,
-	avail_update: snd_pcm_dshare_avail_update,
-	mmap_commit: snd_pcm_dshare_mmap_commit,
+	.status = snd_pcm_dshare_status,
+	.state = snd_pcm_dshare_state,
+	.hwsync = snd_pcm_dshare_hwsync,
+	.delay = snd_pcm_dshare_delay,
+	.prepare = snd_pcm_dshare_prepare,
+	.reset = snd_pcm_dshare_reset,
+	.start = snd_pcm_dshare_start,
+	.drop = snd_pcm_dshare_drop,
+	.drain = snd_pcm_dshare_drain,
+	.pause = snd_pcm_dshare_pause,
+	.rewind = snd_pcm_dshare_rewind,
+	.forward = snd_pcm_dshare_forward,
+	.resume = snd_pcm_dshare_resume,
+	.writei = snd_pcm_mmap_writei,
+	.writen = snd_pcm_mmap_writen,
+	.readi = snd_pcm_dshare_readi,
+	.readn = snd_pcm_dshare_readn,
+	.avail_update = snd_pcm_dshare_avail_update,
+	.mmap_commit = snd_pcm_dshare_mmap_commit,
 };
 
 /**
@@ -684,6 +684,7 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	snd_pcm_direct_t *dshare = NULL;
 	int ret, first_instance;
 	unsigned int chn;
+	int fail_sem_loop = 10;
 
 	assert(pcmp);
 
@@ -716,16 +717,21 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	if (ret < 0)
 		goto _err;
 
-	ret = snd_pcm_direct_semaphore_create_or_connect(dshare);
-	if (ret < 0) {
-		SNDERR("unable to create IPC semaphore");
-		goto _err;
-	}
+	while (1) {
+		ret = snd_pcm_direct_semaphore_create_or_connect(dshare);
+		if (ret < 0) {
+			SNDERR("unable to create IPC semaphore");
+			goto _err;
+		}
 	
-	ret = snd_pcm_direct_semaphore_down(dshare, DIRECT_IPC_SEM_CLIENT);
-	if (ret < 0) {
-		snd_pcm_direct_semaphore_discard(dshare);
-		goto _err;
+		ret = snd_pcm_direct_semaphore_down(dshare, DIRECT_IPC_SEM_CLIENT);
+		if (ret < 0) {
+			snd_pcm_direct_semaphore_discard(dshare);
+			if (--fail_sem_loop <= 0)
+				goto _err;
+			continue;
+		}
+		break;
 	}
 
 	first_instance = ret = snd_pcm_direct_shm_create_or_connect(dshare);
