@@ -590,41 +590,41 @@ static void snd_pcm_dsnoop_dump(snd_pcm_t *pcm, snd_output_t *out)
 }
 
 static snd_pcm_ops_t snd_pcm_dsnoop_ops = {
-	close: snd_pcm_dsnoop_close,
-	info: snd_pcm_dsnoop_info,
-	hw_refine: snd_pcm_dsnoop_hw_refine,
-	hw_params: snd_pcm_dsnoop_hw_params,
-	hw_free: snd_pcm_dsnoop_hw_free,
-	sw_params: snd_pcm_dsnoop_sw_params,
-	channel_info: snd_pcm_dsnoop_channel_info,
-	dump: snd_pcm_dsnoop_dump,
-	nonblock: snd_pcm_dsnoop_nonblock,
-	async: snd_pcm_dsnoop_async,
-	poll_revents: snd_pcm_dsnoop_poll_revents,
-	mmap: snd_pcm_dsnoop_mmap,
-	munmap: snd_pcm_dsnoop_munmap,
+	.close = snd_pcm_dsnoop_close,
+	.info = snd_pcm_dsnoop_info,
+	.hw_refine = snd_pcm_dsnoop_hw_refine,
+	.hw_params = snd_pcm_dsnoop_hw_params,
+	.hw_free = snd_pcm_dsnoop_hw_free,
+	.sw_params = snd_pcm_dsnoop_sw_params,
+	.channel_info = snd_pcm_dsnoop_channel_info,
+	.dump = snd_pcm_dsnoop_dump,
+	.nonblock = snd_pcm_dsnoop_nonblock,
+	.async = snd_pcm_dsnoop_async,
+	.poll_revents = snd_pcm_dsnoop_poll_revents,
+	.mmap = snd_pcm_dsnoop_mmap,
+	.munmap = snd_pcm_dsnoop_munmap,
 };
 
 static snd_pcm_fast_ops_t snd_pcm_dsnoop_fast_ops = {
-	status: snd_pcm_dsnoop_status,
-	state: snd_pcm_dsnoop_state,
-	hwsync: snd_pcm_dsnoop_hwsync,
-	delay: snd_pcm_dsnoop_delay,
-	prepare: snd_pcm_dsnoop_prepare,
-	reset: snd_pcm_dsnoop_reset,
-	start: snd_pcm_dsnoop_start,
-	drop: snd_pcm_dsnoop_drop,
-	drain: snd_pcm_dsnoop_drain,
-	pause: snd_pcm_dsnoop_pause,
-	rewind: snd_pcm_dsnoop_rewind,
-	forward: snd_pcm_dsnoop_forward,
-	resume: snd_pcm_dsnoop_resume,
-	writei: snd_pcm_dsnoop_writei,
-	writen: snd_pcm_dsnoop_writen,
-	readi: snd_pcm_mmap_readi,
-	readn: snd_pcm_mmap_readn,
-	avail_update: snd_pcm_dsnoop_avail_update,
-	mmap_commit: snd_pcm_dsnoop_mmap_commit,
+	.status = snd_pcm_dsnoop_status,
+	.state = snd_pcm_dsnoop_state,
+	.hwsync = snd_pcm_dsnoop_hwsync,
+	.delay = snd_pcm_dsnoop_delay,
+	.prepare = snd_pcm_dsnoop_prepare,
+	.reset = snd_pcm_dsnoop_reset,
+	.start = snd_pcm_dsnoop_start,
+	.drop = snd_pcm_dsnoop_drop,
+	.drain = snd_pcm_dsnoop_drain,
+	.pause = snd_pcm_dsnoop_pause,
+	.rewind = snd_pcm_dsnoop_rewind,
+	.forward = snd_pcm_dsnoop_forward,
+	.resume = snd_pcm_dsnoop_resume,
+	.writei = snd_pcm_dsnoop_writei,
+	.writen = snd_pcm_dsnoop_writen,
+	.readi = snd_pcm_mmap_readi,
+	.readn = snd_pcm_mmap_readn,
+	.avail_update = snd_pcm_dsnoop_avail_update,
+	.mmap_commit = snd_pcm_dsnoop_mmap_commit,
 };
 
 /**
@@ -650,7 +650,7 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 {
 	snd_pcm_t *pcm = NULL, *spcm = NULL;
 	snd_pcm_direct_t *dsnoop = NULL;
-	int ret, first_instance;
+	int ret, first_instance, fail_sem_loop = 10;
 
 	assert(pcmp);
 
@@ -677,16 +677,21 @@ int snd_pcm_dsnoop_open(snd_pcm_t **pcmp, const char *name,
 	if (ret < 0)
 		goto _err;
 
-	ret = snd_pcm_direct_semaphore_create_or_connect(dsnoop);
-	if (ret < 0) {
-		SNDERR("unable to create IPC semaphore");
-		goto _err;
-	}
+	while (1) {
+		ret = snd_pcm_direct_semaphore_create_or_connect(dsnoop);
+		if (ret < 0) {
+			SNDERR("unable to create IPC semaphore");
+			goto _err;
+		}
 	
-	ret = snd_pcm_direct_semaphore_down(dsnoop, DIRECT_IPC_SEM_CLIENT);
-	if (ret < 0) {
-		snd_pcm_direct_semaphore_discard(dsnoop);
-		goto _err;
+		ret = snd_pcm_direct_semaphore_down(dsnoop, DIRECT_IPC_SEM_CLIENT);
+		if (ret < 0) {
+			snd_pcm_direct_semaphore_discard(dsnoop);
+			if (--fail_sem_loop <= 0)
+				goto _err;
+			continue;
+		}
+		break;
 	}
 		
 	first_instance = ret = snd_pcm_direct_shm_create_or_connect(dsnoop);
