@@ -1731,143 +1731,143 @@ static int snd_pcm_hw_rule_sample_bits(snd_pcm_hw_params_t *params,
 
 static snd_pcm_hw_rule_t refine_rules[] = {
 	{
-		var: SND_PCM_HW_PARAM_FORMAT,
-		func: snd_pcm_hw_rule_format,
-		deps: { SND_PCM_HW_PARAM_SAMPLE_BITS, -1 },
-		private_data: 0,
+		.var = SND_PCM_HW_PARAM_FORMAT,
+		.func = snd_pcm_hw_rule_format,
+		.deps = { SND_PCM_HW_PARAM_SAMPLE_BITS, -1 },
+		.private_data = 0,
 	},
 	{
-		var: SND_PCM_HW_PARAM_SAMPLE_BITS, 
-		func: snd_pcm_hw_rule_sample_bits,
-		deps: { SND_PCM_HW_PARAM_FORMAT, 
+		.var = SND_PCM_HW_PARAM_SAMPLE_BITS, 
+		.func = snd_pcm_hw_rule_sample_bits,
+		.deps = { SND_PCM_HW_PARAM_FORMAT, 
 			SND_PCM_HW_PARAM_SAMPLE_BITS, -1 },
-		private_data: 0,
+		.private_data = 0,
 	},
 	{
-		var: SND_PCM_HW_PARAM_SAMPLE_BITS, 
-		func: snd_pcm_hw_rule_div,
-		deps: { SND_PCM_HW_PARAM_FRAME_BITS,
+		.var = SND_PCM_HW_PARAM_SAMPLE_BITS, 
+		.func = snd_pcm_hw_rule_div,
+		.deps = { SND_PCM_HW_PARAM_FRAME_BITS,
 			SND_PCM_HW_PARAM_CHANNELS, -1 },
-		private_data: 0,
+		.private_data = 0,
 	},
 	{
-		var: SND_PCM_HW_PARAM_FRAME_BITS, 
-		func: snd_pcm_hw_rule_mul,
-		deps: { SND_PCM_HW_PARAM_SAMPLE_BITS,
+		.var = SND_PCM_HW_PARAM_FRAME_BITS, 
+		.func = snd_pcm_hw_rule_mul,
+		.deps = { SND_PCM_HW_PARAM_SAMPLE_BITS,
 			SND_PCM_HW_PARAM_CHANNELS, -1 },
-		private_data: 0,
+		.private_data = 0,
 	},
 	{
-		var: SND_PCM_HW_PARAM_FRAME_BITS, 
-		func: snd_pcm_hw_rule_mulkdiv,
-		deps: { SND_PCM_HW_PARAM_PERIOD_BYTES,
+		.var = SND_PCM_HW_PARAM_FRAME_BITS, 
+		.func = snd_pcm_hw_rule_mulkdiv,
+		.deps = { SND_PCM_HW_PARAM_PERIOD_BYTES,
 			SND_PCM_HW_PARAM_PERIOD_SIZE, -1 },
-		private_data: (void*) 8,
+		.private_data = (void*) 8,
 	},
 	{
-		var: SND_PCM_HW_PARAM_FRAME_BITS, 
-		func: snd_pcm_hw_rule_mulkdiv,
-		deps: { SND_PCM_HW_PARAM_BUFFER_BYTES,
+		.var = SND_PCM_HW_PARAM_FRAME_BITS, 
+		.func = snd_pcm_hw_rule_mulkdiv,
+		.deps = { SND_PCM_HW_PARAM_BUFFER_BYTES,
 			SND_PCM_HW_PARAM_BUFFER_SIZE, -1 },
-		private_data: (void*) 8,
+		.private_data = (void*) 8,
 	},
 	{
-		var: SND_PCM_HW_PARAM_CHANNELS, 
-		func: snd_pcm_hw_rule_div,
-		deps: { SND_PCM_HW_PARAM_FRAME_BITS,
+		.var = SND_PCM_HW_PARAM_CHANNELS, 
+		.func = snd_pcm_hw_rule_div,
+		.deps = { SND_PCM_HW_PARAM_FRAME_BITS,
 			SND_PCM_HW_PARAM_SAMPLE_BITS, -1 },
-		private_data: 0,
+		.private_data = 0,
 	},
 	{
-		var: SND_PCM_HW_PARAM_RATE, 
-		func: snd_pcm_hw_rule_mulkdiv,
-		deps: { SND_PCM_HW_PARAM_PERIOD_SIZE,
+		.var = SND_PCM_HW_PARAM_RATE, 
+		.func = snd_pcm_hw_rule_mulkdiv,
+		.deps = { SND_PCM_HW_PARAM_PERIOD_SIZE,
 			SND_PCM_HW_PARAM_PERIOD_TIME, -1 },
-		private_data: (void*) 1000000,
+		.private_data = (void*) 1000000,
 	},
 	{
-		var: SND_PCM_HW_PARAM_RATE, 
-		func: snd_pcm_hw_rule_mulkdiv,
-		deps: { SND_PCM_HW_PARAM_BUFFER_SIZE,
+		.var = SND_PCM_HW_PARAM_RATE, 
+		.func = snd_pcm_hw_rule_mulkdiv,
+		.deps = { SND_PCM_HW_PARAM_BUFFER_SIZE,
 			SND_PCM_HW_PARAM_BUFFER_TIME, -1 },
-		private_data: (void*) 1000000,
+		.private_data = (void*) 1000000,
 	},
 	{
-		var: SND_PCM_HW_PARAM_PERIODS, 
-		func: snd_pcm_hw_rule_div,
-		deps: { SND_PCM_HW_PARAM_BUFFER_SIZE,
+		.var = SND_PCM_HW_PARAM_PERIODS, 
+		.func = snd_pcm_hw_rule_div,
+		.deps = { SND_PCM_HW_PARAM_BUFFER_SIZE,
 			SND_PCM_HW_PARAM_PERIOD_SIZE, -1 },
-		private_data: 0,
+		.private_data = 0,
 	},
 	{
-		var: SND_PCM_HW_PARAM_PERIOD_SIZE, 
-		func: snd_pcm_hw_rule_div,
-		deps: { SND_PCM_HW_PARAM_BUFFER_SIZE,
+		.var = SND_PCM_HW_PARAM_PERIOD_SIZE, 
+		.func = snd_pcm_hw_rule_div,
+		.deps = { SND_PCM_HW_PARAM_BUFFER_SIZE,
 			SND_PCM_HW_PARAM_PERIODS, -1 },
-		private_data: 0,
+		.private_data = 0,
 	},
 	{
-		var: SND_PCM_HW_PARAM_PERIOD_SIZE, 
-		func: snd_pcm_hw_rule_mulkdiv,
-		deps: { SND_PCM_HW_PARAM_PERIOD_BYTES,
+		.var = SND_PCM_HW_PARAM_PERIOD_SIZE, 
+		.func = snd_pcm_hw_rule_mulkdiv,
+		.deps = { SND_PCM_HW_PARAM_PERIOD_BYTES,
 			SND_PCM_HW_PARAM_FRAME_BITS, -1 },
-		private_data: (void*) 8,
+		.private_data = (void*) 8,
 	},
 	{
-		var: SND_PCM_HW_PARAM_PERIOD_SIZE, 
-		func: snd_pcm_hw_rule_muldivk,
-		deps: { SND_PCM_HW_PARAM_PERIOD_TIME,
+		.var = SND_PCM_HW_PARAM_PERIOD_SIZE, 
+		.func = snd_pcm_hw_rule_muldivk,
+		.deps = { SND_PCM_HW_PARAM_PERIOD_TIME,
 			SND_PCM_HW_PARAM_RATE, -1 },
-		private_data: (void*) 1000000,
+		.private_data = (void*) 1000000,
 	},
 	{
-		var: SND_PCM_HW_PARAM_BUFFER_SIZE, 
-		func: snd_pcm_hw_rule_mul,
-		deps: { SND_PCM_HW_PARAM_PERIOD_SIZE,
+		.var = SND_PCM_HW_PARAM_BUFFER_SIZE, 
+		.func = snd_pcm_hw_rule_mul,
+		.deps = { SND_PCM_HW_PARAM_PERIOD_SIZE,
 			SND_PCM_HW_PARAM_PERIODS, -1 },
-		private_data: 0,
+		.private_data = 0,
 	},
 	{
-		var: SND_PCM_HW_PARAM_BUFFER_SIZE, 
-		func: snd_pcm_hw_rule_mulkdiv,
-		deps: { SND_PCM_HW_PARAM_BUFFER_BYTES,
+		.var = SND_PCM_HW_PARAM_BUFFER_SIZE, 
+		.func = snd_pcm_hw_rule_mulkdiv,
+		.deps = { SND_PCM_HW_PARAM_BUFFER_BYTES,
 			SND_PCM_HW_PARAM_FRAME_BITS, -1 },
-		private_data: (void*) 8,
+		.private_data = (void*) 8,
 	},
 	{
-		var: SND_PCM_HW_PARAM_BUFFER_SIZE, 
-		func: snd_pcm_hw_rule_muldivk,
-		deps: { SND_PCM_HW_PARAM_BUFFER_TIME,
+		.var = SND_PCM_HW_PARAM_BUFFER_SIZE, 
+		.func = snd_pcm_hw_rule_muldivk,
+		.deps = { SND_PCM_HW_PARAM_BUFFER_TIME,
 			SND_PCM_HW_PARAM_RATE, -1 },
-		private_data: (void*) 1000000,
+		.private_data = (void*) 1000000,
 	},
 	{
-		var: SND_PCM_HW_PARAM_PERIOD_BYTES, 
-		func: snd_pcm_hw_rule_muldivk,
-		deps: { SND_PCM_HW_PARAM_PERIOD_SIZE,
+		.var = SND_PCM_HW_PARAM_PERIOD_BYTES, 
+		.func = snd_pcm_hw_rule_muldivk,
+		.deps = { SND_PCM_HW_PARAM_PERIOD_SIZE,
 			SND_PCM_HW_PARAM_FRAME_BITS, -1 },
-		private_data: (void*) 8,
+		.private_data = (void*) 8,
 	},
 	{
-		var: SND_PCM_HW_PARAM_BUFFER_BYTES, 
-		func: snd_pcm_hw_rule_muldivk,
-		deps: { SND_PCM_HW_PARAM_BUFFER_SIZE,
+		.var = SND_PCM_HW_PARAM_BUFFER_BYTES, 
+		.func = snd_pcm_hw_rule_muldivk,
+		.deps = { SND_PCM_HW_PARAM_BUFFER_SIZE,
 			SND_PCM_HW_PARAM_FRAME_BITS, -1 },
-		private_data: (void*) 8,
+		.private_data = (void*) 8,
 	},
 	{
-		var: SND_PCM_HW_PARAM_PERIOD_TIME, 
-		func: snd_pcm_hw_rule_mulkdiv,
-		deps: { SND_PCM_HW_PARAM_PERIOD_SIZE,
+		.var = SND_PCM_HW_PARAM_PERIOD_TIME, 
+		.func = snd_pcm_hw_rule_mulkdiv,
+		.deps = { SND_PCM_HW_PARAM_PERIOD_SIZE,
 			SND_PCM_HW_PARAM_RATE, -1 },
-		private_data: (void*) 1000000,
+		.private_data = (void*) 1000000,
 	},
 	{
-		var: SND_PCM_HW_PARAM_BUFFER_TIME, 
-		func: snd_pcm_hw_rule_mulkdiv,
-		deps: { SND_PCM_HW_PARAM_BUFFER_SIZE,
+		.var = SND_PCM_HW_PARAM_BUFFER_TIME, 
+		.func = snd_pcm_hw_rule_mulkdiv,
+		.deps = { SND_PCM_HW_PARAM_BUFFER_SIZE,
 			SND_PCM_HW_PARAM_RATE, -1 },
-		private_data: (void*) 1000000,
+		.private_data = (void*) 1000000,
 	},
 };
 
@@ -1875,64 +1875,64 @@ static snd_pcm_hw_rule_t refine_rules[] = {
 
 static snd_mask_t refine_masks[SND_PCM_HW_PARAM_LAST_MASK - SND_PCM_HW_PARAM_FIRST_MASK + 1] = {
 	[SND_PCM_HW_PARAM_ACCESS - SND_PCM_HW_PARAM_FIRST_MASK] = {
-		bits: { 0x1f },
+		.bits = { 0x1f },
 	},
 	[SND_PCM_HW_PARAM_FORMAT - SND_PCM_HW_PARAM_FIRST_MASK] = {
-		bits: { 0x81ffffff, 0xfff},
+		.bits = { 0x81ffffff, 0xfff},
 	},
 	[SND_PCM_HW_PARAM_SUBFORMAT - SND_PCM_HW_PARAM_FIRST_MASK] = {
-		bits: { 0x1 },
+		.bits = { 0x1 },
 	},
 };
   
 static snd_interval_t refine_intervals[SND_PCM_HW_PARAM_LAST_INTERVAL - SND_PCM_HW_PARAM_FIRST_INTERVAL + 1] = {
 	[SND_PCM_HW_PARAM_SAMPLE_BITS - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 1, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 1, empty: 0,
+		.min = 1, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 1, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_FRAME_BITS - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 1, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 1, empty: 0,
+		.min = 1, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 1, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_CHANNELS - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 1, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 1, empty: 0,
+		.min = 1, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 1, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_RATE - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 1, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 0, empty: 0,
+		.min = 1, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 0, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_PERIOD_TIME - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 0, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 0, empty: 0,
+		.min = 0, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 0, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_PERIOD_SIZE - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 0, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 0, empty: 0,
+		.min = 0, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 0, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_PERIOD_BYTES - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 0, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 0, empty: 0,
+		.min = 0, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 0, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_PERIODS - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 0, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 0, empty: 0,
+		.min = 0, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 0, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_BUFFER_TIME - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 1, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 0, empty: 0,
+		.min = 1, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 0, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_BUFFER_SIZE - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 1, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 1, empty: 0,
+		.min = 1, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 1, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_BUFFER_BYTES - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 1, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 1, empty: 0,
+		.min = 1, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 1, .empty = 0,
 	},
 	[SND_PCM_HW_PARAM_TICK_TIME - SND_PCM_HW_PARAM_FIRST_INTERVAL] = {
-		min: 0, max: UINT_MAX,
-		openmin: 0, openmax: 0, integer: 0, empty: 0,
+		.min = 0, .max = UINT_MAX,
+		.openmin = 0, .openmax = 0, .integer = 0, .empty = 0,
 	},
 };
 
