@@ -646,25 +646,25 @@ int snd_pcm_plugin_hw_params_slave(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 }
 
 snd_pcm_fast_ops_t snd_pcm_plugin_fast_ops = {
-	status: snd_pcm_plugin_status,
-	state: snd_pcm_plugin_state,
-	hwsync: snd_pcm_plugin_hwsync,
-	delay: snd_pcm_plugin_delay,
-	prepare: snd_pcm_plugin_prepare,
-	reset: snd_pcm_plugin_reset,
-	start: snd_pcm_plugin_start,
-	drop: snd_pcm_plugin_drop,
-	drain: snd_pcm_plugin_drain,
-	pause: snd_pcm_plugin_pause,
-	rewind: snd_pcm_plugin_rewind,
-	forward: snd_pcm_plugin_forward,
-	resume: snd_pcm_plugin_resume,
-	writei: snd_pcm_plugin_writei,
-	writen: snd_pcm_plugin_writen,
-	readi: snd_pcm_plugin_readi,
-	readn: snd_pcm_plugin_readn,
-	avail_update: snd_pcm_plugin_avail_update,
-	mmap_commit: snd_pcm_plugin_mmap_commit,
+	.status = snd_pcm_plugin_status,
+	.state = snd_pcm_plugin_state,
+	.hwsync = snd_pcm_plugin_hwsync,
+	.delay = snd_pcm_plugin_delay,
+	.prepare = snd_pcm_plugin_prepare,
+	.reset = snd_pcm_plugin_reset,
+	.start = snd_pcm_plugin_start,
+	.drop = snd_pcm_plugin_drop,
+	.drain = snd_pcm_plugin_drain,
+	.pause = snd_pcm_plugin_pause,
+	.rewind = snd_pcm_plugin_rewind,
+	.forward = snd_pcm_plugin_forward,
+	.resume = snd_pcm_plugin_resume,
+	.writei = snd_pcm_plugin_writei,
+	.writen = snd_pcm_plugin_writen,
+	.readi = snd_pcm_plugin_readi,
+	.readn = snd_pcm_plugin_readn,
+	.avail_update = snd_pcm_plugin_avail_update,
+	.mmap_commit = snd_pcm_plugin_mmap_commit,
 };
 
 
