@@ -722,19 +722,19 @@ static void snd_pcm_route_dump(snd_pcm_t *pcm, snd_output_t *out)
 }
 
 static snd_pcm_ops_t snd_pcm_route_ops = {
-	close: snd_pcm_route_close,
-	info: snd_pcm_plugin_info,
-	hw_refine: snd_pcm_route_hw_refine,
-	hw_params: snd_pcm_route_hw_params,
-	hw_free: snd_pcm_plugin_hw_free,
-	sw_params: snd_pcm_plugin_sw_params,
-	channel_info: snd_pcm_plugin_channel_info,
-	dump: snd_pcm_route_dump,
-	nonblock: snd_pcm_plugin_nonblock,
-	async: snd_pcm_plugin_async,
-	poll_revents: snd_pcm_plugin_poll_revents,
-	mmap: snd_pcm_plugin_mmap,
-	munmap: snd_pcm_plugin_munmap,
+	.close = snd_pcm_route_close,
+	.info = snd_pcm_plugin_info,
+	.hw_refine = snd_pcm_route_hw_refine,
+	.hw_params = snd_pcm_route_hw_params,
+	.hw_free = snd_pcm_plugin_hw_free,
+	.sw_params = snd_pcm_plugin_sw_params,
+	.channel_info = snd_pcm_plugin_channel_info,
+	.dump = snd_pcm_route_dump,
+	.nonblock = snd_pcm_plugin_nonblock,
+	.async = snd_pcm_plugin_async,
+	.poll_revents = snd_pcm_plugin_poll_revents,
+	.mmap = snd_pcm_plugin_mmap,
+	.munmap = snd_pcm_plugin_munmap,
 };
 
 static int route_load_ttable(snd_pcm_route_params_t *params, snd_pcm_stream_t stream,
