@@ -574,40 +574,40 @@ static void snd_pcm_shm_dump(snd_pcm_t *pcm, snd_output_t *out)
 }
 
 static snd_pcm_ops_t snd_pcm_shm_ops = {
-	close: snd_pcm_shm_close,
-	info: snd_pcm_shm_info,
-	hw_refine: snd_pcm_shm_hw_refine,
-	hw_params: snd_pcm_shm_hw_params,
-	hw_free: snd_pcm_shm_hw_free,
-	sw_params: snd_pcm_shm_sw_params,
-	channel_info: snd_pcm_shm_channel_info,
-	dump: snd_pcm_shm_dump,
-	nonblock: snd_pcm_shm_nonblock,
-	async: snd_pcm_shm_async,
-	mmap: snd_pcm_shm_mmap,
-	munmap: snd_pcm_shm_munmap,
+	.close = snd_pcm_shm_close,
+	.info = snd_pcm_shm_info,
+	.hw_refine = snd_pcm_shm_hw_refine,
+	.hw_params = snd_pcm_shm_hw_params,
+	.hw_free = snd_pcm_shm_hw_free,
+	.sw_params = snd_pcm_shm_sw_params,
+	.channel_info = snd_pcm_shm_channel_info,
+	.dump = snd_pcm_shm_dump,
+	.nonblock = snd_pcm_shm_nonblock,
+	.async = snd_pcm_shm_async,
+	.mmap = snd_pcm_shm_mmap,
+	.munmap = snd_pcm_shm_munmap,
 };
 
 static snd_pcm_fast_ops_t snd_pcm_shm_fast_ops = {
-	status: snd_pcm_shm_status,
-	state: snd_pcm_shm_state,
-	hwsync: snd_pcm_shm_hwsync,
-	delay: snd_pcm_shm_delay,
-	prepare: snd_pcm_shm_prepare,
-	reset: snd_pcm_shm_reset,
-	start: snd_pcm_shm_start,
-	drop: snd_pcm_shm_drop,
-	drain: snd_pcm_shm_drain,
-	pause: snd_pcm_shm_pause,
-	rewind: snd_pcm_shm_rewind,
-	forward: snd_pcm_shm_forward,
-	resume: snd_pcm_shm_resume,
-	writei: snd_pcm_mmap_writei,
-	writen: snd_pcm_mmap_writen,
-	readi: snd_pcm_mmap_readi,
-	readn: snd_pcm_mmap_readn,
-	avail_update: snd_pcm_shm_avail_update,
-	mmap_commit: snd_pcm_shm_mmap_commit,
+	.status = snd_pcm_shm_status,
+	.state = snd_pcm_shm_state,
+	.hwsync = snd_pcm_shm_hwsync,
+	.delay = snd_pcm_shm_delay,
+	.prepare = snd_pcm_shm_prepare,
+	.reset = snd_pcm_shm_reset,
+	.start = snd_pcm_shm_start,
+	.drop = snd_pcm_shm_drop,
+	.drain = snd_pcm_shm_drain,
+	.pause = snd_pcm_shm_pause,
+	.rewind = snd_pcm_shm_rewind,
+	.forward = snd_pcm_shm_forward,
+	.resume = snd_pcm_shm_resume,
+	.writei = snd_pcm_mmap_writei,
+	.writen = snd_pcm_mmap_writen,
+	.readi = snd_pcm_mmap_readi,
+	.readn = snd_pcm_mmap_readn,
+	.avail_update = snd_pcm_shm_avail_update,
+	.mmap_commit = snd_pcm_shm_mmap_commit,
 };
 
 static int make_local_socket(const char *filename)
